/*
 * Cay S. Horstmann & Gary Cornell, Core Java
 * Published By Sun Microsystems Press/Prentice-Hall
 * Copyright (C) 1997 Sun Microsystems Inc.
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this 
 * software and its documentation for NON-COMMERCIAL purposes
 * and without fee is hereby granted provided that this 
 * copyright notice appears in all copies. 
 * 
 * THE AUTHORS AND PUBLISHER MAKE NO REPRESENTATIONS OR 
 * WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHORS
 * AND PUBLISHER SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING 
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
/**
 * @version 1.00 10 Sep 1997
 * @author Cay Horstmann
 */

package corejava;

import java.security.*;

public class CaesarCipher extends Cipher
{  public CaesarCipher() { super("CAESAR"); }
   
   protected int engineBlockSize() { return 1; }

   protected int engineOutBufferSize(int inLen, boolean last)
   { return inLen; }
   
   protected void engineInitEncrypt(Key k) 
   {  if (k instanceof CaesarKey)
         key = (CaesarKey)k;
      else throw new IllegalArgumentException
         ("Not a CAESAR key: " + k);
   }

   protected void engineInitDecrypt(Key k) 
   {  if (k instanceof CaesarKey)
         key = (CaesarKey)k;
      else throw new IllegalArgumentException
         ("Not a CAESAR key: " + k);
   }

   protected int engineUpdate(byte in[], int inOff, int inLen,
      byte out[], int outOff)
   {  int k = key.getKeyValue();
      if (getState() == Cipher.UNINITIALIZED) return 0;
      if (getState() == Cipher.DECRYPT) k = -k;
      for (int i = 0; i < inLen; i++)
      {  out[i + outOff] = (byte)(in[i + inOff] + k);
      }
      return inLen;
   }

   CaesarKey key;
}
